" Vim indent file
" Language:		Ruby
" Maintainer:		Doug Kearns <dougkearns@gmail.com>
" Info:			$Id: eruby.vim,v 1.4 2006/04/15 20:19:04 vimboss Exp $
" URL:			http://vim-ruby.rubyforge.org
" Anon CVS:		See above site
" Release Coordinator:	Doug Kearns <dougkearns@gmail.com>

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
  finish
endif

runtime! indent/html.vim

" vim: nowrap sw=2 sts=2 ts=8 ff=unix:
